# TECHMAN ROBOT INC. ("Techman") CONFIDENTIAL
# Copyright (C) 2020~2024 Techman. All Rights Reserved.
# This source code and any compilation or derivative thereof is proprietary and confidential to Techman.
# Under no circumstances may any portion of the source code or any modified version of the source code be distributed,
# disclosed or otherwise made available to any third party without the express written consent of Techman.
# Author: Thomas Zhao
# Verson: 2.14.1000


from flask import Flask, request, jsonify, g
from werkzeug.exceptions import HTTPException
from waitress import serve
from cv2 import cv2

import numpy as np
import datetime
import time
import socket


app = Flask(__name__)
HOST_NAME = 'TM Vision HTTP Server'
HOST_PORT = 4585


# ============================== SYSTEM ==============================
@app.errorhandler(HTTPException)
def handleException(e):
    '''Return HTTP errors.'''
    TRIMessage(e)
    return e

@app.before_request
def before_request_func():
    TRIMessage(f'[{request.environ["REMOTE_ADDR"]}:{request.environ["REMOTE_PORT"]}] -> {request.environ["REQUEST_METHOD"]}({request.environ["PATH_INFO"]}) - start\ndata: {request.data}\nform: {request.form}\nvalues: {request.values}\njson: {request.json}')
    g.start = time.time()

@app.after_request
def after_request_func(response):
    TRIMessage(f'Completed in {time.time() - g.start:.3f} secs')
    TRIMessage(response.data)
    TRIMessage(f'[{request.environ["REMOTE_ADDR"]}:{request.environ["REMOTE_PORT"]}] -> {request.environ["REQUEST_METHOD"]}({request.environ["PATH_INFO"]}) - end')
    return response

def TRIMessage(message):
    print(f'\n[{datetime.datetime.now(datetime.timezone(datetime.timedelta(0))).astimezone().isoformat(timespec="milliseconds")}] {message}')

# ============================== GET ==============================
@app.route('/', methods=['GET'])
def default():
    # user defined method
    result = {
        "result": "api",
        "message": "running",
    } 
    return jsonify(result)

@app.route('/api/<string:m_method>', methods=['GET'])
def get(m_method):
    # user defined method
    if m_method == 'status':
        result = {
            "result": "status",
            "message": "im ok"
        }
    else:
        result = {
            "result": "fail",
            "message": "wrong request"
        }
    return jsonify(result)

# ============================== POST ==============================
@app.route('/api/<string:m_method>', methods=['POST'])
def post(m_method):
    # get key/value
    parameters = request.form
    model_id = parameters.get('model_id')
    TRIMessage(f'model_id: {model_id}')

    # check key/value
    if model_id is None:
        TRIMessage('model_id is not set')
        result={                    
            "message": "fail",
            "result": "model_id required"
        }
        return jsonify(result)

    # convert image data
    img = cv2.imdecode(np.fromstring(request.files['image'].read(), np.uint8), cv2.IMREAD_UNCHANGED)

    # Clsssification
    if m_method == 'CLS':
        # do inference here

        result = {
            "message": "success",
            "result": "NG",
            "score": 0.987
        }
    # Detection
    elif m_method == 'DET':
        # do inference here

        result = {
            "message":"success",
            "annotations": 
            [
                { 
                    "box_cx": 150,
                    "box_cy": 150,
                    "box_w": 100,
                    "box_h": 100,
                    "label": "apple",
                    "score": 0.964,
                    "rotation": -45
                },
                { 
                    "box_cx": 550,
                    "box_cy": 550,
                    "box_w": 100,
                    "box_h": 100,
                    "label": "car",
                    "score": 1.000,
                    "rotation": 0
                },
                { 
                    "box_cx": 350,
                    "box_cy": 350,
                    "box_w": 150,
                    "box_h": 150,
                    "label": "mobilephone",
                    "score": 0.886,
                    "rotation": 135
                }
            ],
            "result": None
        }
    # no method
    else:
        result = {
            "message": "no method",
            "result": None
        }

    return jsonify(result)

# ============================== ENTRY POINT ==============================
if __name__ == '__main__':
    check=False
    try:
        host_addr = ([ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")] or [[(s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close()) for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]])
        check = True if len(host_addr) > 0 else False
    except Exception as e:
        TRIMessage(e)
    if check == True:
        host_addr = host_addr[-1] if len(host_addr) > 1 else host_addr[0]
        TRIMessage(f'serving on http://{host_addr}:{HOST_PORT}')
    else:
        TRIMessage(f'serving on http://127.0.0.1:{HOST_PORT}')
    serve(app, port=HOST_PORT, ident=HOST_NAME, _quiet=True)
